/* ----------------------------------------------------------------------
scriptio.c : read esf array test script
John O'Donnell, May 2013
---------------------------------------------------------------------- */

#include <stdio.h>
//#include "esftypes.h"

#define OPERATIONSLIMIT   100000
#define OPSDATASIZE       8*OPERATIONSLIMIT

#define CODEINITIALIZE    101
#define CODETERMINATE     102
#define CODEUPDATE        103
#define CODEDELETE        104
#define CODELOOKUP        105

#define RESULT_OK                   0
#define RESULT_NOTHING              1
#define RESULT_NONEXISTENT_ARRAY    2
#define RESULT_NONEXISTENT_ELEMENT  3
#define RESULT_MEMORY_FULL          4


/* ---------------------------------------------------------------------- */

void printoperations (int*ops, int opstop)
{ int i = 0;
  while (i<opstop)
    { printf ("%7d %7d %7d %7d %7d %7d %7d %7d\n",
	      ops[i], ops[i+1], ops[i+2], ops[i+3],
	      ops[i+4], ops[i+5], ops[i+6], ops[i+7]);
      i += 8;
    }
}


/* Read a line of input, fill 7 entries in the ops array starting at
   location j.  Returns 0 if end of input, 1 otherwise. */

int readoperation (int *ops, int j)
{
  int r; 
  char opletter;
  char c;
  char token [8];

  int opnum = 0;
  int arr = 0;
  int idx = 0;
  int val = 0;
  int nfy = 0;
  int expresultcode = 0;
  int expresultval = 0;

  /*  printf("\n\nEntering readoperation\n"); */

  r = scanf ("%d.%1s", &opnum, &opletter);

/*  printf ("r = %d\n", r);
    printf ("opnum = %d, opletter=(%c)\n",
    opnum, opletter); */

  if (opletter=='I')
    { /* printf ("Initializen"); */
      ops[j++] = opnum;
      ops[j++] = CODEINITIALIZE;
      ops[j++] = 0;
      ops[j++] = 0;
      ops[j++] = 0;
      ops[j++] = 0;
      ops[j++] = RESULT_OK;
      ops[j++] = 0;
      return 1;
    }

  if (opletter=='T')
    { /* printf ("Terminate\n"); */
      ops[j++] = opnum;
      ops[j++] = CODETERMINATE;
      ops[j++] = 0;
      ops[j++] = 0;
      ops[j++] = 0;
      ops[j++] = 0;
      ops[j++] = RESULT_OK;
      ops[j++] = 0;
      return 0;  /* indicate that this is the last operation */
    }

  if (opletter=='D')
    { /* printf ("Delete\n"); */
      r = scanf ("%d", &arr);

      ops[j++] = opnum;
      ops[j++] = CODEDELETE;
      ops[j++] = arr;
      ops[j++] = 0;
      ops[j++] = 0;
      ops[j++] = 0;
      ops[j++] = RESULT_OK;
      ops[j++] = 0;
      return 1;
    }

  if (opletter=='U')
    { /* printf ("Update\n"); */
      r = scanf ("%d [%d] = %d %d => %s", &arr, &idx, &val, &nfy, &token);
      /*      printf ("reading U,  r = %d, arr = %d, idx = %d, tok=%s\n",
	      r, arr, idx, &token); */
      c = token[0];
      if (c=='N')
        { /* printf ("L, reading Nothing"); */
	  expresultcode = RESULT_NOTHING;
          expresultval = 0;
        }
      else
        { /* printf ("L, reading Just\n"); */
	  r = scanf ("%d", &expresultval);
	  expresultcode = RESULT_OK;
        }
/* printf ("Lookup: expresultcode = %d, expresultval = %d\n", 
            expresultcode, expresultval); */

      ops[j++] = opnum;
      ops[j++] = CODEUPDATE;
      ops[j++] = arr;
      ops[j++] = idx;
      ops[j++] = val;
      ops[j++] = nfy;
      ops[j++] = expresultcode;
      ops[j++] = expresultval;
      return 1;
    }

  if (opletter=='L')
    { /* printf ("Lookup\n"); */
      r = scanf (" %d [%d] => %s", &arr, &idx, &token);
/* printf ("reading L, r = %d, idx = %d, tok=%s\n", r, idx, &token); */
      c = token[0];
      if (c=='N')
        { /* printf ("L, reading Nothing"); */
	  expresultcode = RESULT_NOTHING;
          expresultval = 0;
        }
      else
        { /* printf ("L, reading Just\n"); */
	  r = scanf ("%d", &expresultval);
	  expresultcode = RESULT_OK;
        }
/* printf ("Lookup: expresultcode = %d, expresultval = %d\n", 
            expresultcode, expresultval); */

      ops[j++] = opnum;
      ops[j++] = CODELOOKUP;
      ops[j++] = arr;
      ops[j++] = idx;
      ops[j++] = val;
      ops[j++] = 0;
      ops[j++] = expresultcode;
      ops[j++] = expresultval;
      return 1;
    }
}
/* ---------------------------------------------------------------------- */

/* Read a text file containing the operations, decode them and store in the
   array ops. */

int readoperations (int *ops)
{
  int i = 0;
  int loc = 0;

  /*  printf ("readoperations\n"); */
  int more = 1;
  while (more)
    { /* printf ("readoperations loop:  loc = %d\n", loc); */
       more = readoperation(ops,loc);
      loc += 8;
   }
  return loc;
}
